@@ -62,6 +62,7 @@ module Data.Conduit.Combinators
     , foldlE
     , foldMap
     , foldMapE
+    , foldWhile
     , all
     , allE
     , any
@@ -1755,6 +1756,19 @@ mapAccumWhileC f =
 {-# INLINE mapAccumWhileC #-}
 STREAMING(mapAccumWhile, mapAccumWhileC, mapAccumWhileS, f s)
 
+
+-- | Specialized version of 'mapAccumWhile' that does not provide values downstream.
+--
+-- @since 1.3.4
+foldWhile :: Monad m => (a -> s -> Either e s) -> s -> ConduitT a o m (Either e s)
+foldWhile f = loop
+  where
+    loop !s = await >>= maybe (return $ Right s) go
+      where
+        go a = either (return . Left $!) loop $ f a s
+{-# INLINE foldWhile #-}
+
+
 -- | 'concatMap' with an accumulator.
 --
 -- Subject to fusion
@@ -1819,7 +1833,7 @@ chunksOfExactlyE :: (Monad m, Seq.IsSequence seq) => Seq.Index seq -> ConduitT s
 chunksOfExactlyE chunkSize = await >>= maybe (return ()) start
     where
         start b
-            | onull b = chunksOfE chunkSize
+            | onull b = chunksOfExactlyE chunkSize
             | Seq.lengthIndex b < chunkSize = continue (Seq.lengthIndex b) [b]
             | otherwise = let (first,rest) = Seq.splitAt chunkSize b in
                             yield first >> start rest
