@@ -38,6 +38,8 @@ module Data.Conduit.Internal.Conduit
     , runConduitRes
     , fuse
     , connect
+    , unconsM
+    , unconsEitherM
       -- ** Composition
     , connectResume
     , connectResumeConduit
@@ -106,7 +108,7 @@ import Data.Void (Void, absurd)
 import Data.Monoid (Monoid (mappend, mempty))
 import Data.Semigroup (Semigroup ((<>)))
 import Control.Monad.Trans.Resource
-import Data.Conduit.Internal.Pipe hiding (yield, mapOutput, leftover, yieldM, await, awaitForever, bracketP)
+import Data.Conduit.Internal.Pipe hiding (yield, mapOutput, leftover, yieldM, await, awaitForever, bracketP, unconsM, unconsEitherM)
 import qualified Data.Conduit.Internal.Pipe as CI
 import Control.Monad (forever)
 import Data.Traversable (Traversable (..))
@@ -326,8 +328,8 @@ connectResume (SealedConduitT left0) (ConduitT right0) =
         recurse = goLeft rp rc
 
 sourceToPipe :: Monad m => Source m o -> Pipe l i o u m ()
-sourceToPipe =
-    go . flip unConduitT Done
+sourceToPipe (ConduitT k) =
+    go $ k Done
   where
     go (HaveOutput p o) = HaveOutput (go p) o
     go (NeedInput _ c) = go $ c ()
@@ -336,8 +338,8 @@ sourceToPipe =
     go (Leftover p ()) = go p
 
 sinkToPipe :: Monad m => Sink i m r -> Pipe l i o u m r
-sinkToPipe =
-    go . injectLeftovers . flip unConduitT Done
+sinkToPipe (ConduitT k) =
+    go $ injectLeftovers $ k Done
   where
     go (HaveOutput _ o) = absurd o
     go (NeedInput p c) = NeedInput (go . p) (const $ go $ c ())
@@ -346,8 +348,8 @@ sinkToPipe =
     go (Leftover _ l) = absurd l
 
 conduitToPipe :: Monad m => Conduit i m o -> Pipe l i o u m ()
-conduitToPipe =
-    go . injectLeftovers . flip unConduitT Done
+conduitToPipe (ConduitT k) =
+    go $ injectLeftovers $ k Done
   where
     go (HaveOutput p o) = HaveOutput (go p) o
     go (NeedInput p c) = NeedInput (go . p) (const $ go $ c ())
@@ -358,7 +360,7 @@ conduitToPipe =
 -- | Generalize a 'Source' to a 'Producer'.
 --
 -- Since 1.0.0
-toProducer :: Monad m => Source m a -> Producer m a
+toProducer :: Monad m => Source m a -> ConduitT i a m ()
 toProducer (ConduitT c0) = ConduitT $ \rest -> let
     go (HaveOutput p o) = HaveOutput (go p) o
     go (NeedInput _ c) = go (c ())
@@ -399,8 +401,9 @@ catchC :: (MonadUnliftIO m, Exception e)
        -> ConduitT i o m r
 catchC (ConduitT p0) onErr = ConduitT $ \rest -> let
     go (Done r) = rest r
-    go (PipeM mp) = PipeM $ withRunInIO $ \run -> E.catch (run (liftM go mp))
-        (return . flip unConduitT rest . onErr)
+    go (PipeM mp) = PipeM $ withRunInIO $ \ run ->
+      run (liftM go mp) `E.catch` \ e ->
+        return $ onErr e `unConduitT` rest
     go (Leftover p i) = Leftover (go p) i
     go (NeedInput x y) = NeedInput (go . x) (go . y)
     go (HaveOutput p o) = HaveOutput (go p) o
@@ -691,8 +694,8 @@ passthroughSink (ConduitT sink0) final = ConduitT $ \rest -> let
 --
 -- Since 1.2.6
 sourceToList :: Monad m => Source m a -> m [a]
-sourceToList =
-    go . flip unConduitT Done
+sourceToList (ConduitT k) =
+    go $ k Done
   where
     go (Done _) = return []
     go (HaveOutput src x) = liftM (x:) (go src)
@@ -720,6 +723,40 @@ connect :: Monad m
         -> m r
 connect = ($$)
 
+-- | Split a conduit into head and tail.
+--
+-- Note that you have to 'sealConduitT' it first.
+--
+-- Since 1.3.3
+unconsM :: Monad m
+        => SealedConduitT () o m ()
+        -> m (Maybe (o, SealedConduitT () o m ()))
+unconsM (SealedConduitT p) = go p
+  where
+    -- This function is the same as @Pipe.unconsM@ but it ignores leftovers.
+    go (HaveOutput p o) = pure $ Just (o, SealedConduitT p)
+    go (NeedInput _ c) = go $ c ()
+    go (Done ()) = pure Nothing
+    go (PipeM mp) = mp >>= go
+    go (Leftover p ()) = go p
+
+-- | Split a conduit into head and tail or return its result if it is done.
+--
+-- Note that you have to 'sealConduitT' it first.
+--
+-- Since 1.3.3
+unconsEitherM :: Monad m
+              => SealedConduitT () o m r
+              -> m (Either r (o, SealedConduitT () o m r))
+unconsEitherM (SealedConduitT p) = go p
+  where
+    -- This function is the same as @Pipe.unconsEitherM@ but it ignores leftovers.
+    go (HaveOutput p o) = pure $ Right (o, SealedConduitT p)
+    go (NeedInput _ c) = go $ c ()
+    go (Done r) = pure $ Left r
+    go (PipeM mp) = mp >>= go
+    go (Leftover p ()) = go p
+
 -- | Named function synonym for '.|'
 --
 -- Equivalent to '.|' and '=$='. However, the latter is
